use std::env;
use dotenv::dotenv;
use url::form_urlencoded::Serializer;
use sha1::{Sha1, Digest};
use std::fs;
use std::collections::HashSet;

fn sha1(input : &str) -> String {
        let mut sha = Sha1::new();
        sha.input(input);
        format!("{:x}", sha.result())
}

fn checksum (method : &str, query : &str) -> String {
  let bbb_api_secret : String = env::var("BBB_API_SECRET").unwrap();
  sha1(&format!("{}{}{}", method, query, &bbb_api_secret))
}

fn build_api_url(endpoint : &str, params : &[(&str, &str)]) -> String {

  let bbb_api_url : String = env::var("BBB_API_URL").unwrap();
  let api = if bbb_api_url.ends_with('/') { "api/" } else { "/api/" };

  let mut ser = Serializer::new(String::new());
  for (k, v) in params.iter() {
    ser.append_pair(k, v);
  }
  let params : String = ser.finish();
  let hash = checksum(endpoint, &params);
  let url_base = format!("{}{}{}?{}", bbb_api_url, api, endpoint, params);
  format!("{}&checksum={}", url_base, hash)
}

fn main() {
    dotenv().unwrap();

    let re = regex::Regex::new(r"^[0-9a-f]{40}-\d+$").unwrap();
    let meetings_on_disk : HashSet<String> = fs::read_dir("/var/bigbluebutton/").unwrap().filter_map(|e| {
      let as_string = e.unwrap().file_name().to_str().unwrap().to_owned();
      if re.is_match(&as_string) {
        Some(as_string)
      } else {
        None
      }
    }).collect();

    println!("found {} meetings on disk.", meetings_on_disk.len());

    let get_meetings_url = build_api_url("getMeetings", &[]);
    let document = reqwest::blocking::get(&get_meetings_url).unwrap().text().unwrap();
    let re = regex::Regex::new(r"<internalMeetingID>([0-9a-f]{40}-\d+)</internalMeetingID>").unwrap();
    let mut running_meetings : HashSet<String> = HashSet::new();
    for id in re.captures_iter(&document) {
      running_meetings.insert(id[1].to_owned());
    }

    println!("found {} running meetings.", running_meetings.len());

    let stale_meetings : Vec<String> = meetings_on_disk.difference(&running_meetings).map(|x|{format!("/var/bigbluebutton/{}", x)}).collect();

    println!("found {} stale meetings.", stale_meetings.len());

    for dirname in stale_meetings {
      print!("deleting {}...", dirname);
      std::fs::remove_dir_all(dirname).unwrap();
      println!(" removed!");
    }

    println!("done cleaning up.");
}
